//-*-Mode:C++;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
// ex: set ft=cpp fenc=utf-8 sts=4 ts=4 sw=4 et:
//
// BSD LICENSE
// 
// Copyright (c) 2009-2011, Michael Truog <mjtruog at gmail dot com>
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     * All advertising materials mentioning features or use of this
//       software must display the following acknowledgment:
//         This product includes software developed by Michael Truog
//     * The name of the author may not be used to endorse or promote
//       products derived from this software without specific prior
//       written permission
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//

#include "timer.hpp"

#if HAVE_CLOCK_GETTIME_MONOTONIC

timer::timer()
{
    ::clock_gettime(CLOCK_MONOTONIC, &m_start);
}

void timer::restart()
{
    ::clock_gettime(CLOCK_MONOTONIC, &m_start);
}

double timer::elapsed() const
{
    struct timespec end;
    ::clock_gettime(CLOCK_MONOTONIC, &end);
    return (static_cast<double>(end.tv_sec - m_start.tv_sec) +
            static_cast<double>(end.tv_nsec - m_start.tv_nsec) * 1.0e-9);
}

#else

timer::timer()
{
    ::gettimeofday(&m_start, 0);
}

void timer::restart()
{
    ::gettimeofday(&m_start, 0);
}

double timer::elapsed() const
{
    struct timeval end;
    ::gettimeofday(&end, 0);
    return (static_cast<double>(end.tv_sec - m_start.tv_sec) +
            static_cast<double>(end.tv_usec - m_start.tv_usec) * 1.0e-6);
}

#endif

